@@ -58,6 +58,10 @@ constexpr unsigned char ASCII_CODES_DIGIT_VALUES[256] = {
     37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37, 37,
     37, 37, 37, 37, 37, 37, 37, 37, 37,
 };
+constexpr char DIGIT_VALUES_ASCII_CODES[MAX_REPRESENTABLE_BASE] = {
+    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b',
+    'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
+    'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'};
 
 static constexpr unsigned char mask_char(char character) {
   return static_cast<unsigned char>(character & 0xff);
@@ -215,7 +219,7 @@ class BigInt {
   static_assert(_BINARY_SHIFT <= std::numeric_limits<SignedDigit>::digits - 1,
                 "Digit should be able to hold all integers lesser than double "
                 "signed base.");
-  static constexpr Digit BINARY_SHIFT = _BINARY_SHIFT;
+  static constexpr std::size_t BINARY_SHIFT = _BINARY_SHIFT;
 
   using DoubleDigit = double_precision_t<Digit>;
   static_assert(!std::is_same<DoubleDigit, undefined>(),
@@ -423,13 +427,13 @@ class BigInt {
       if (iterations_count == 0) {
         if (smallest_digits_count == 1) {
           std::vector<Digit> quotient;
-          Digit remainder = divmod_digits_by_digit(
+          Digit remainder = divrem_digits_by_digit(
               largest_digits, smallest_digits[0], quotient);
           largest_digits = smallest_digits;
           smallest_digits = std::vector<Digit>({remainder});
         } else {
           std::vector<Digit> quotient, remainder;
-          divmod_two_or_more_digits(largest_digits, smallest_digits, quotient,
+          divrem_two_or_more_digits(largest_digits, smallest_digits, quotient,
                                     remainder);
           largest_digits = smallest_digits;
           smallest_digits = remainder;
@@ -688,13 +692,13 @@ class BigInt {
     normalize_digits(quotient_digits);
     if (divisor_digits_count == 1) {
       std::vector<Digit> next_quotient_digits;
-      Digit remainder = divmod_digits_by_digit(
+      Digit remainder = divrem_digits_by_digit(
           quotient_digits, divisor_digits[0], next_quotient_digits);
       std::swap(quotient_digits, next_quotient_digits);
       if (remainder) inexact = true;
     } else {
       std::vector<Digit> next_quotient_digits, remainder;
-      divmod_two_or_more_digits(quotient_digits, divisor_digits,
+      divrem_two_or_more_digits(quotient_digits, divisor_digits,
                                 next_quotient_digits, remainder);
       std::swap(quotient_digits, next_quotient_digits);
       if (remainder.size() > 1 || remainder[0] != 0) inexact = true;
@@ -804,7 +808,7 @@ class BigInt {
             std::size_t TARGET_SHIFT = floor_log<BASE>(BINARY_BASE),
             std::size_t TARGET_BASE = power(BASE, TARGET_SHIFT)>
   std::string repr() const {
-    static_assert(1 < BASE && BASE < MAX_REPRESENTABLE_BASE,
+    static_assert(1 < BASE && BASE <= MAX_REPRESENTABLE_BASE,
                   "Base should be range from 2 to 36.");
     const std::vector<Digit> base_digits = to_base_digits<TARGET_BASE>();
     const std::size_t characters_count =
@@ -815,13 +819,13 @@ class BigInt {
     for (std::size_t index = 0; index < base_digits.size() - 1; index++) {
       Digit remainder = base_digits[index];
       for (std::size_t step = 0; step < TARGET_SHIFT; step++) {
-        *--stop = '0' + (remainder % BASE);
+        *--stop = DIGIT_VALUES_ASCII_CODES[remainder % BASE];
         remainder /= BASE;
       }
     }
     for (Digit remainder = base_digits.back(); remainder != 0;
          remainder /= BASE)
-      *--stop = '0' + (remainder % BASE);
+      *--stop = DIGIT_VALUES_ASCII_CODES[remainder % BASE];
     if (_sign == 0)
       *--stop = '0';
     else if (_sign < 0)
@@ -855,8 +859,6 @@ class BigInt {
   int _sign;
   std::vector<Digit> _digits;
 
-  static constexpr Digit KARATSUBA_CUTOFF = 70;
-  static constexpr Digit KARATSUBA_SQUARE_CUTOFF = KARATSUBA_CUTOFF * 2;
   static constexpr std::size_t MANTISSA_BITS =
       std::numeric_limits<double>::digits;
   static constexpr double MANTISSA_BITS_POWER_OF_TWO =
@@ -873,15 +875,15 @@ class BigInt {
   static_assert(WINDOW_SHIFT <= std::numeric_limits<WindowDigit>::digits,
                 "Window digit type should be able to contain window digits.");
 
-  static void divmod_two_or_more_digits(const std::vector<Digit>& dividend,
+  static void divrem_two_or_more_digits(const std::vector<Digit>& dividend,
                                         const std::vector<Digit>& divisor,
                                         std::vector<Digit>& quotient,
                                         std::vector<Digit>& remainder) {
     std::size_t dividend_digits_count = dividend.size();
-    std::size_t divisor_digits_count = divisor.size();
-    Digit* dividend_normalized = new Digit[dividend_digits_count + 1]();
-    Digit* divisor_normalized = new Digit[divisor_digits_count]();
-    std::size_t shift = BINARY_SHIFT - to_bit_length(divisor.back());
+    const std::size_t divisor_digits_count = divisor.size();
+    Digit* const dividend_normalized = new Digit[dividend_digits_count + 1]();
+    Digit* const divisor_normalized = new Digit[divisor_digits_count]();
+    const std::size_t shift = BINARY_SHIFT - to_bit_length(divisor.back());
     shift_digits_left(divisor.data(), divisor_digits_count, shift,
                       divisor_normalized);
     Digit accumulator = shift_digits_left(
@@ -889,11 +891,12 @@ class BigInt {
     if (accumulator != 0 || dividend_normalized[dividend_digits_count - 1] >=
                                 divisor_normalized[divisor_digits_count - 1])
       dividend_normalized[dividend_digits_count++] = accumulator;
-    std::size_t quotient_size = dividend_digits_count - divisor_digits_count;
-    Digit* quotient_data = new Digit[quotient_size]();
-    Digit last_divisor_digit_normalized =
+    const std::size_t quotient_size =
+        dividend_digits_count - divisor_digits_count;
+    Digit* const quotient_data = new Digit[quotient_size]();
+    const Digit last_divisor_digit_normalized =
         divisor_normalized[divisor_digits_count - 1];
-    Digit penult_divisor_digit_normalized =
+    const Digit penult_divisor_digit_normalized =
         divisor_normalized[divisor_digits_count - 2];
     for (Digit *digits_normalized_tail = dividend_normalized + quotient_size,
                *quotient_position = quotient_data + quotient_size;
@@ -917,11 +920,11 @@ class BigInt {
       }
       SignedDigit accumulator = 0;
       for (std::size_t index = 0; index < divisor_digits_count; ++index) {
-        SignedDoubleDigit step;
-        step = static_cast<SignedDigit>(digits_normalized_tail[index]) +
-               accumulator -
-               static_cast<SignedDoubleDigit>(quotient_digit) *
-                   static_cast<SignedDoubleDigit>(divisor_normalized[index]);
+        const SignedDoubleDigit step =
+            static_cast<SignedDigit>(digits_normalized_tail[index]) +
+            accumulator -
+            static_cast<SignedDoubleDigit>(quotient_digit) *
+                static_cast<SignedDoubleDigit>(divisor_normalized[index]);
         digits_normalized_tail[index] =
             static_cast<Digit>(step) & BINARY_DIGIT_MASK;
         accumulator = static_cast<SignedDigit>(step >> BINARY_SHIFT);
@@ -955,7 +958,7 @@ class BigInt {
     normalize_digits(remainder);
   }
 
-  static Digit divmod_digits_by_digit(const std::vector<Digit>& dividend,
+  static Digit divrem_digits_by_digit(const std::vector<Digit>& dividend,
                                       Digit divisor,
                                       std::vector<Digit>& quotient) {
     DoubleDigit remainder = 0;
@@ -1114,10 +1117,13 @@ class BigInt {
       std::swap(shortest, longest);
       std::swap(size_shortest, size_longest);
     }
+    static constexpr std::size_t KARATSUBA_CUTOFF = 70;
+    static constexpr std::size_t KARATSUBA_SQUARE_CUTOFF = KARATSUBA_CUTOFF * 2;
     if (size_shortest <=
         ((shortest == longest) ? KARATSUBA_SQUARE_CUTOFF : KARATSUBA_CUTOFF)) {
-      return size_shortest == 0 ? std::vector<Digit>({0})
-                                : multiply_digits_plain(*shortest, *longest);
+      return size_shortest == 1 && (*shortest)[0] == 0
+                 ? std::vector<Digit>({0})
+                 : multiply_digits_plain(*shortest, *longest);
     }
     if (2 * size_shortest <= size_longest)
       return multiply_digits_lopsided(*shortest, *longest);
@@ -1185,11 +1191,10 @@ class BigInt {
     std::vector<Digit> result(first_size + second_size, 0);
     if (&first == &second)
       for (std::size_t index = 0; index < first_size; ++index) {
-        DoubleDigit accumulator;
         DoubleDigit digit = first[index];
         auto result_position = result.begin() + (index << 1);
         auto first_position = first.begin() + (index + 1);
-        accumulator = *result_position + digit * digit;
+        DoubleDigit accumulator = *result_position + digit * digit;
         *(result_position++) =
             static_cast<Digit>(accumulator & BINARY_DIGIT_MASK);
         accumulator >>= BINARY_SHIFT;
@@ -1263,17 +1268,17 @@ class BigInt {
       int remainder_sign = _sign;
       if (divisor_digits_count == 1) {
         std::vector<Digit> quotient_digits;
-        Digit remainder_digit = divmod_digits_by_digit(
+        Digit remainder_digit = divrem_digits_by_digit(
             _digits, divisor._digits[0], quotient_digits);
         remainder_sign *= remainder_digit != 0;
         if constexpr (WITH_QUOTIENT)
           *quotient = BigInt(_sign * divisor._sign, quotient_digits);
         if constexpr (WITH_REMAINDER)
           *remainder =
-              BigInt(_sign * static_cast<SignedDigit>(remainder_digit));
+              BigInt(remainder_sign, std::vector<Digit>{remainder_digit});
       } else {
         std::vector<Digit> quotient_digits, remainder_digits;
-        divmod_two_or_more_digits(_digits, divisor._digits, quotient_digits,
+        divrem_two_or_more_digits(_digits, divisor._digits, quotient_digits,
                                   remainder_digits);
         remainder_sign *=
             remainder_digits.size() > 1 || remainder_digits[0] != 0;
@@ -1329,7 +1334,7 @@ class BigInt {
       DoubleDigit step =
           (static_cast<DoubleDigit>(input_digits[index]) << shift) |
           accumulator;
-      output_digits[index] = static_cast<Digit>(step) & BINARY_DIGIT_MASK;
+      output_digits[index] = static_cast<Digit>(step & BINARY_DIGIT_MASK);
       accumulator = static_cast<Digit>(step >> BINARY_SHIFT);
     }
     return accumulator;
@@ -1339,11 +1344,11 @@ class BigInt {
                                   std::size_t input_digits_count,
                                   std::size_t shift, Digit* output_digits) {
     Digit accumulator = 0;
-    Digit mask = (static_cast<Digit>(1) << shift) - 1;
+    DoubleDigit mask = (static_cast<DoubleDigit>(1) << shift) - 1;
     for (std::size_t index = input_digits_count; index-- > 0;) {
       DoubleDigit step = static_cast<DoubleDigit>(accumulator) << BINARY_SHIFT |
                          input_digits[index];
-      accumulator = static_cast<Digit>(step) & mask;
+      accumulator = static_cast<Digit>(step & mask);
       output_digits[index] = static_cast<Digit>(step >> shift);
     }
     return accumulator;
@@ -1438,8 +1443,8 @@ class BigInt {
         ++start;
         continue;
       }
-      DoubleDigit digit =
-          static_cast<Digit>(ASCII_CODES_DIGIT_VALUES[mask_char(*(start++))]);
+      DoubleDigit digit = static_cast<DoubleDigit>(
+          ASCII_CODES_DIGIT_VALUES[mask_char(*(start++))]);
       std::size_t base_exponent = 1;
       for (; base_exponent < infimum_base_exponent && start != stop; ++start) {
         if (*start == SEPARATOR) continue;
